@@ -19,7 +19,7 @@ use RDF::Query::Stream;
 our ($VERSION, $debug);
 BEGIN {
 	$debug		= 0;
-	$VERSION	= do { my $REV = (qw$Revision: 166 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= do { my $REV = (qw$Revision: 175 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
 }
 
 ######################################################################
@@ -39,10 +39,13 @@ sub new {
 	my $model	= shift;
 	my %args	= @_;
 	
-	unless (UNIVERSAL::isa($model, 'RDF::Redland::Model')) {
+	if (not defined $model) {
 		my $storage	= RDF::Redland::Storage->new( "hashes", "test", "new='yes',hash-type='memory',contexts='yes'" );
 		$model	= RDF::Redland::Model->new( $storage, '' );
 	}
+
+	throw RDF::Query::Error::MethodInvocationError ( -text => 'Not a RDF::Redland::Model passed to bridge constructor' ) unless (blessed($model) and $model->isa('RDF::Redland::Model'));
+	
 	my $self	= bless( {
 					model	=> $model,
 					parsed	=> $args{parsed},
@@ -92,9 +95,6 @@ sub new_literal {
 		push(@args, $type);
 	} elsif ($lang) {
 		push(@args, undef);
-	}
-	
-	if ($lang) {
 		push(@args, $lang);
 	}
 	
@@ -394,7 +394,11 @@ sub get_statements {
 	if ($context) {
 		my $iter	= $model->find_statements( $stmt, $context );
 		$args{ context }	= $context;
+
+		my $finished	= 0;
 		$stream	= sub {
+			$finished	= 1 if (@_ and $_[0] eq 'close');
+			return undef if ($finished);
 			if (@_ and $_[0] eq 'context') {
 				return $context;
 			} elsif (not $iter) {
@@ -430,7 +434,10 @@ sub get_statements {
 			}
 			my $iter	= $model->$imethod( @defs );
 			my $context;
+			my $finished	= 0;
 			$stream	= sub {
+				$finished	= 1 if (@_ and $_[0] eq 'close');
+				return undef if ($finished);
 				if (@_ and $_[0] eq 'context') {
 					return $context;
 				} elsif (not $iter) {
@@ -450,8 +457,11 @@ sub get_statements {
 		} else {
 			my $iter	= $model->find_statements( $stmt );
 			warn "iterator: $iter (" . $stmt->as_string . ')' if (0);
+			my $finished	= 0;
 			my $context;
 			$stream	= sub {
+				$finished	= 1 if (@_ and $_[0] eq 'close');
+				return undef if ($finished);
 				no warnings 'uninitialized';
 				if (@_ and $_[0] eq 'context') {
 					return $context;
@@ -537,13 +547,18 @@ Possible features include:
 
 	* named_graph
 	* node_counts
+	* temp_model
+	* xml
 
 =cut
 
 sub supports {
 	my $self	= shift;
 	my $feature	= shift;
+	
+	return 1 if ($feature eq 'temp_model');
 	return 1 if ($feature eq 'named_graph');
+	return 1 if ($feature eq 'xml');
 #	return 1 if ($feature eq 'node_counts');	# XXX just used for testing -- redland doesn't have efficient statement counting. see C<node_count> below.
 	return 0;
 }
